﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;

using Midi;

namespace RB3KB_USB2MIDI {
    public partial class FormMain : Form {

        private static OutputDevice MIDI_OUT = null;
        private static Channel channel = Channel.Channel1;
        private static USBKB keyboard = null;

        enum PedalMode { Expression, ChannelVolume, FootController };

        public FormMain() {
            InitializeComponent();
        }

        private void FormMain_Load(object sender, EventArgs e) {
            foreach (OutputDevice device in OutputDevice.InstalledDevices) {
                ddlMidiOut.Items.Add(device.Name);
            }
        }

        private void Start() {
            MIDI_OUT.Open();
            keyboard = new USBKB();
        }

        private void End() {
            keyboard.Close();
            MIDI_OUT.Close();
        }

        private void USB2MIDI() {
            //bool DrumMapping = false;
            //int Program = 1;
            int Octave = 4;

            bool queueOctaveUp = false;
            bool queueOctaveDown = false;

            KBData previous = keyboard.Read();
            while (true) {
                KBData data = keyboard.Read();

                /*
                //Not ready
                if (data.DpadUp) DrumMapping = !DrumMapping;
                if (data.Triangle2) Program++;
                else if (data.CrossA) Program--;
                if (Program < 1) Program = 1;
                if (Program > 128) Program = 128;
                */

                #region Octave
                if (data.CircleB) {
                    queueOctaveUp = true;
                } else if (queueOctaveUp) {
                    queueOctaveUp = false;
                    Octave++;
                    Console.WriteLine(Octave);
                }

                if (data.Square1) {
                    queueOctaveDown = true;
                } else if(queueOctaveDown) {
                    queueOctaveDown = false;
                    Octave--;
                    Console.WriteLine(Octave);
                }

                if (Octave > 8)
                    Octave = 8;
                else if (Octave < 0)
                    Octave = 0;
                #endregion

                for (int i = 0; i < data.Key.Length; i++) {
                    if (data.Key[i] && !previous.Key[i]) {
                        int pitch = (Octave * 12) + i;
                        int velocity = 80;
                        MIDI_OUT.SendNoteOn(channel, (Pitch)pitch, velocity);
                    } else if (!data.Key[i] && previous.Key[i]) {
                        int pitch = (Octave * 12) + i;
                        int velocity = 0;
                        MIDI_OUT.SendNoteOff(channel, (Pitch)pitch, velocity);
                    }
                }

                previous = data;
            }
        }

        private void ddlMidiOut_SelectedIndexChanged(object sender, EventArgs e) {
            MIDI_OUT = OutputDevice.InstalledDevices[ddlMidiOut.SelectedIndex];
        }

        Thread thread = null;
        private void btnRunLoop_Click(object sender, EventArgs e) {
            if (thread == null || !thread.IsAlive) {
                if (MIDI_OUT != null) {
                    Start();

                    if (keyboard.CanUse()) {
                        thread = new Thread(() => USB2MIDI());
                        thread.Start();

                        btnRunLoop.Text = "Running";
                    } else {
                        End();
                        btnRunLoop.Text = "Keyboard not found - missing filter?";
                    }
                }
            } else {
                thread.Abort();
                End();

                btnRunLoop.Text = "Doing Nothing";
            }
            
        }
    }
}
